import hashlib
import json
import os
import shutil
import tarfile

def write_env_file(envs, file_path):
    file_dir = os.path.dirname(file_path)
    if not os.path.exists(file_dir):
        os.makedirs(file_dir)
    with open(file_path, "w") as f:
        for key, value in envs.items():
            f.write("export {}={}\n".format(key, value))
    return True


def write_pod_file(pods, file_path):
    file_dir = os.path.dirname(file_path)
    if not os.path.exists(file_dir):
        os.makedirs(file_dir)
    with open(file_path, "w") as f:
        f.write(json.dumps(pods) + '\n')
    return True


def write_tempest_conf_file(tempest_conf, file_path):
    file_dir = os.path.dirname(file_path)
    if not os.path.exists(file_dir):
        os.makedirs(file_dir)
    with open(file_path, "w") as f:
        f.write(json.dumps(tempest_conf) + '\n')
    return True


def copy_vm_images(src_dir, des_dir):
    if not os.path.exists(des_dir):
        os.makedirs(des_dir)
    for f in os.listdir(src_dir):
        src_file = os.path.join(src_dir, f)
        dest_file = os.path.join(des_dir, f)
        if os.path.isfile(src_file):
            shutil.copy(src_file, dest_file)
    return True


def get_execution_status(results_dir, testcases, requestId):
    results = []
    for tc in testcases:
        if tc.startswith('functest'):
            status, result = get_functest_status(results_dir, tc)
            res = {'testCaseName': tc, 'testSuiteName': 'function',
                   'scenario': 'nfvi', 'executionId': requestId,
                   'results': result, 'status': status}
            if not result:
                res['timestart'] = None
                res['endTime'] = None
            else:
                res['timestart'] = result['timestart']
                res['endTime'] = result['timestop']
            results.append(res)
        if tc.startswith('yardstick'):
            status, result = get_yardstick_status(results_dir, tc)
            res = {'testCaseName': tc, 'testSuiteName': 'function',
                   'scenario': 'nfvi', 'executionId': requestId,
                   'results': result, 'status': status,
                   'timestart': None, 'endTime': None}
            results.append(res)
    return results


def get_functest_status(results_dir, testcase):
    res_file = os.path.join(results_dir, 'functest_results.txt')
    total_file = os.path.join(results_dir, 'results.json')
    if not os.path.isfile(res_file):
        if not os.path.isfile(total_file):
            return 'IN_PROGRESS', None
        return 'FAILED', None
    criteria = None
    sub_testcase = []
    timestart = None
    timestop = None
    if os.path.isfile(total_file):
        with open(total_file, 'r') as f:
            for jsonfile in f:
                try:
                    data = json.loads(jsonfile)
                    for item in data['testcases_list']:
                        if item['name'] == testcase:
                            criteria = item['result']
                            sub_testcase = item['sub_testcase']
                            break
                    else:
                        return 'FAILED', None
                except KeyError as e:
                    return 'FAILED', None
                except ValueError:
                    continue
    with open(res_file, 'r') as f:
        for jsonfile in f:
            try:
                data = json.loads(jsonfile)
                if data['build_tag'].endswith(testcase):
                    criteria = data['criteria'] if not criteria else criteria
                    timestart = data['start_date']
                    timestop = data['stop_date']
                    if criteria == 'FAIL':
                        return 'IN_PROGRESS', None
                    break
            except KeyError as e:
                return 'FAILED', None
            except ValueError:
                continue
        else:
            if not criteria:
                return 'IN_PROGRESS', None


    status = 'COMPLETED' if criteria == 'PASS' else 'FAILED'
    results = {'criteria': criteria, 'sub_testcase': sub_testcase,
               'timestart': timestart, 'timestop': timestop}
    return status, results


def get_yardstick_status(results_dir, testcase):
    res_file = os.path.join(results_dir, 'ha_logs', '{}.out'.format(testcase))
    total_file = os.path.join(results_dir, 'results.json')
    if not os.path.isfile(res_file):
        if not os.path.isfile(total_file):
            return 'IN_PROGRESS', None
        return 'FAILED', None
    with open(res_file, 'r') as f:
        for jsonfile in f:
            data = json.loads(jsonfile)
            try:
                criteria = data['result']['criteria']
                if criteria == 'PASS':
                    details = data['result']['testcases']
                    for key, value in details.items():
                        sla_pass = value['tc_data'][0]['data']['sla_pass']
                        if not 1 == sla_pass:
                            criteria = 'FAIL'
            except KeyError as e:
                return 'FAILED {}'.format(e), None

    status = 'COMPLETED' if criteria == 'PASS' else 'FAILED'
    results = {'criteria': criteria, 'timestart': None, 'timestop': None}
    return status, results


def clean_workspace(dovetail_home):
    pre_config = os.path.join(dovetail_home, 'pre_config')
    images = os.path.join(dovetail_home, 'images')
    patches = os.path.join(dovetail_home, 'patches')
    rm_dirs = [pre_config, images, patches]
    for item in rm_dirs:
        if os.path.exists(item):
            shutil.rmtree(item)


def clean_home(dovetail_home):
    if os.path.exists(dovetail_home):
        shutil.rmtree(dovetail_home)


def _get_checksum(path):
    checksum = hashlib.sha256()

    if os.path.isfile(path):
        with open(path, 'rb') as f:
            for chunk in iter(lambda: f.read(4096), b''):
                checksum.update(chunk)

    return checksum.hexdigest()


def package_vtp_results(vtp_home, exec_id):
    result_path = os.path.join(vtp_home, 'results')
    if not os.path.exists(result_path):
        os.makedirs(result_path)
    result_file = os.path.join(vtp_home, 'vtp-' + exec_id + '.txt')
    report_obj = {}

    if not os.path.isfile(result_file):
        msg = 'Could not find file {}.'.format(result_file)
        return False, msg

    dest_path = os.path.join(result_path, 'onap-vtp_logs')
    if not os.path.exists(dest_path):
        os.makedirs(dest_path)
    shutil.copy(result_file, dest_path)
    with open(os.path.join(result_path, 'dovetail.log'), 'w') as f:
        f.write("This is generated by CVC Test Center.")
    with open(result_file) as f:
        try:
            data = json.load(f)
            for item in data['data']:
                csar_file = item['parameters']['csar']
                report_obj['test_date'] = item['startTime']
                if not item['status'].lower() == 'completed':
                    result = 'FAIL'
                    break
            else:
                result = 'PASS'
            report_obj['vnf_checksum'] = _get_checksum(csar_file)
        except KeyError as e:
            msg = 'KeyError: {}.'.format(e)
            return False, msg
        except ValueError as e:
            msg = 'Failed to load json data, {}.'.format(e)
            return False, msg
        except Exception:
            msg = 'Error.'
            return False, msg
    report_obj['build_tag'] = 'daily-master-{}'.format(exec_id)
    report_obj['version'] = '2019.04'
    report_obj['vnf_type'] = 'tosca'
    testcases_list = [{'objective': 'onap csar validation', 'sub_testcase': [],
                       'mandatory': True, 'name': 'onap-vtp.validate.csar', 'result': result}]
    report_obj['testcases_list'] = testcases_list
    try:
        with open(os.path.join(result_path, 'results.json'), 'w') as f:
            f.write(json.dumps(report_obj) + '\n')
        tar_file = os.path.join(vtp_home, 'logs_{}.tar.gz'.format(exec_id))
        cwd = os.getcwd()
        os.chdir(vtp_home)
        with tarfile.open(tar_file, 'w:gz') as f_out:
            files = os.listdir(result_path)
            for f in files:
                if f not in ['workspace']:
                    f_out.add(os.path.join('results', f))
        os.chdir(cwd)
        msg = 'Success to package files to {}.'.format(tar_file)
        return True, msg
    except Exception:
        msg = 'Failed to package files.'
        return False, msg
            
